import logging
import optparse
import imp
import sys

from locale import gettext as _

from quickly import quicklyconfig
from quickly.core.commands import CommandError

LEVELS = (  logging.ERROR,
            logging.WARNING,
            logging.INFO,
            logging.DEBUG,
            )

def main():
    version = quicklyconfig.__version__
    # Support for command line options.
    usage = _("quickly [command] [options]")
    parser = optparse.OptionParser(version="%%prog %s" % version, usage=usage)
    parser.add_option('-d', '--debug', dest='debug_mode', action='store_true',
        help=_('Print the maximum debugging info (implies -vv)'))
    parser.add_option('-v', '--verbose', dest='logging_level', action='count',
        help=_('set error_level output to warning, info, and then debug'))

    parser.set_defaults(logging_level=0, foo=None)
    (options, args) = parser.parse_args()

    # set the verbosity
    if options.debug_mode:
        options.logging_level = 3
    logging.basicConfig(level=LEVELS[options.logging_level], format='%(asctime)s %(levelname)s %(message)s')

    if len(args) >= 1:
        command = args.pop(0)
    else:
        logging.error(_('No Command provided'))
        sys.exit(1)
    
    # Create is a special case because we specify the template after
    # the command, rather than implicily from the project
    if len(args) >= 1 and command == 'create':
        template = args.pop(0)
        logging.debug(_('creating with template: %s') % template)
    else:
        logging.error("TODO: implement template lookup from project")
        sys.exit(1)

    try:
        sys.path.append(quicklyconfig.get_data_file('templates'))
        logging.debug(_('Looking for template: %s' % template))
        template_mod = imp.load_module(template, *imp.find_module(template))
        template_instance = template_mod.Template()
        logging.debug(_('Found template in %s' % template_instance.get_template_root()))
    except ImportError:
        logging.error(_('Failed to load template: %s' % template))
        sys.exit(1)
        
    logging.debug(_('%s icon: %s') % (template, template_instance._meta.icon))
    logging.debug(_('%s commands: %s') % (template, template_instance.get_all_commands()))
        
    # Run your cli application there.
    logging.debug(_('Looking for command: %s' % command))
    cmd = template_instance.get_command(command)
    logging.debug(_('Command icon: %s' % cmd._meta.icon))
    logging.debug(_('Running command: %s' % cmd._meta.name))
    try:
        cmd.run(args)
    except CommandError, e:
        logging.error(_('Failure in %s command "%s": %s' % (template, command, e.message)))
        sys.exit(1)


if __name__ == "__main__":
    main()
