# Base command classes

class CommandBuilder(type):
    """
    Metaclass for all Commands.
    """
    def __new__(cls, name, bases, attrs):
        super_new = super(CommandBuilder, cls).__new__
        parents = [b for b in bases if isinstance(b, CommandBuilder)]
        if not parents:
            # If this isn't a subclass of TemplateBuilder, don't do anything special.
            return super_new(cls, name, bases, attrs)

        # Create the class.
        module = attrs.pop('__module__')
        new_class = super_new(cls, name, bases, {'__module__': module})

        if 'Meta' in attrs:
            default_options = attrs.pop('Meta')
        else:
            default_options = {}
            
        options = CommandConfig(cls, name, module, default_options)

        for aName, a in attrs.items():
            setattr(new_class, aName, a)
        setattr(new_class, '_meta', options)
        return new_class
    

class CommandConfig(object):
    
    def __init__(self, cls, cls_name, module, default_options=None):
        self.name = getattr(default_options, 'name', cls_name.replace('_', ' ').title())
        self.icon = getattr(default_options, 'icon', cls_name.lower())
        self.description = getattr(default_options, 'description', 'Describe what the command is for')

class BaseCommand(object):
    __metaclass__ = CommandBuilder

    def __init__(self, name=None):
        self.name = name or self._meta.name
    
    def _add_to_template(self, template):
        self.template = template
        
    def run(self, args=None):
        # Run this command
        pass

class CommandError(Exception):
    
    def __init__(self, cmd, msg):
        super(CommandError, self).__init__(msg)
        self.command = cmd
