# Example project class

from quickly.core.templates import BaseTemplate

class BaseProject(object):
    def __init__(self, project_name, project_path):
        self.project_name = project_name
        self.project_path = project_path
        
        self.project_config = dict()
        self.template_name = None
        self.lp_name = None

    def _load_config(self):
        fileconfig = file(os.path.join(self.project_path, '.quickly'), 'rb')
        for line in fileconfig: 
            fields = line.split('#')[0] # Suppress commentary after the value in configuration file and in full line
            fields = fields.split('=', 1) # Separate variable from value
            # normally, we have two fields in "fields"
            if len(fields) == 2:
                self.project_config[fields[0].strip()] = fields[1].strip() 
        fileconfig.close()
        
        if 'template' in self.project_config:
            self.template_name = self.project_config['template']

        if 'lp_id' in self.project_config:
            self.lp_name = self.project_config['lp_id']

    def get_icon(self):
        icon_file = '%s/%s.svg' % (self.project_path, self.project_name)
        return icon_file

    def get_template(self):
        # Should return an instance of this project's template
        return BaseTemplate(self.template_name)
    
    def get_lp_project(self):
        return self.lp_name
    
    def get_commands(self):
        return self.get_template().get_available_commands(self)
    
    def get_command(self, name):
        for command in self.get_commands():
            if command._meta.name == name:
                return command
        return None

