# Base template classes
from quickly.core.commands import BaseCommand

class TemplateBuilder(type):
    """
    Metaclass for all Templates.
    """
    def __new__(cls, name, bases, attrs):
        super_new = super(TemplateBuilder, cls).__new__
        parents = [b for b in bases if isinstance(b, TemplateBuilder)]
        if not parents:
            # If this isn't a subclass of TemplateBuilder, don't do anything special.
            return super_new(cls, name, bases, attrs)

        # Create the class.
        module = attrs.pop('__module__')
        new_class = super_new(cls, name, bases, {'__module__': module})

        if 'Meta' in attrs:
            default_options = attrs.pop('Meta')
        else:
            default_options = {}
            
        options = TemplateConfig(cls, name, module, default_options)
        setattr(new_class, '_meta', options)

        commands = {}
        for aName, a in attrs.items():
            if isinstance(a, BaseCommand):
                commands[aName] = a
            else:
                setattr(new_class, aName, a)
        setattr(new_class, '_commands', commands)
        return new_class
    

class TemplateConfig(object):
    
    def __init__(self, cls, cls_name, module, default_options=None):
        self.name = getattr(default_options, 'name', module.replace('_', ' ').title())
        self.icon = getattr(default_options, 'icon', module)
        self.description = getattr(default_options, 'description', 'Describe what the template is for')

class BaseTemplate(object):
    __metaclass__ = TemplateBuilder

    def __init__(self, name=None):
        self.name = name or self._meta.name
        for cmd in self._commands.values():
            cmd._add_to_template(self)
        
    def get_template_root(self):
        import os
        return os.path.dirname(__file__)

    def get_command(self, cmd_name):
        return self._commands.get(cmd_name, None)
    
    def get_all_commands(self):
        # Return a list of commands for this template
        return self._commands.keys()
        
    def get_available_commands(self, project):
        # Get commands currently available for the given project
        return self._commands.keys()
        
